import { nls } from "zen-lingual";
import { waitForIt } from "zen-cerebellum";
import { MESSAGE_NOTIFY, MESSAGE_DISMISS, MESSAGES_CLEAR } from "../constants";

/** FSA : generic message logging
  @param {bool} sticky - true if fixed/persistent message
  @param {string} message - text message to log
  @param {string} title - title of log message
  @param {string} level - gravity level (success, warning, danger, info)
  @param {object} options - for extra-customization
*/
function logMessage(sticky, message, title = "", level = "info", options = {}) {
  return {
    type: MESSAGE_NOTIFY,
    payload: {
      //basic message guid..
      mid: Date.now(),
      message,
      title,
      level,
      sticky,
      options
    }
  };
}

/**
 * Factory for creating generic log methods.
 * @param {bool} sticky - true if fixed/persistent message
 * @return {func} Thenable function following message display.
 */
const logMessageFactory = sticky => (...p) => dispatch => {
  //create message
  const log = logMessage(sticky, ...p);

  return waitForIt(resolve => {
    //inject resolver into payload to be resolved when user confirm/cancel
    log.payload.resolve = resolve;
    dispatch(log);
  });
};

/**
 * FSA : Display a modal alert message.
 */
export const prompt = logMessageFactory(true);

/**
 * FSA : Show a temporary notification.
 */
export const notify = logMessageFactory(false);

/**
 * FSA : Dismiss a unique message notification.
 * @param {object} message - logged message
 * @param {object} reason - dismiss reason, false if message has been cancelled
 */
export const dismissMessage = (message, reason = false) => dispatch => {
  //mark message has resolved first !
  message.resolve(reason);
  return dispatch({
    type: MESSAGE_DISMISS,
    payload: message
  });
};

/**
 * FSA : Clear all message notifications
 */
export const clearMessages = () => ({
  type: MESSAGES_CLEAR
});

//mapping from back-end exception level to gravity level
const messageLevelMap = {
  "1": "info",
  "2": "warning",
  "3": "danger"
};

/** FSA : notify an internal error
  @param error (object) : XHR failure error    
*/
export function logInternalError(error) {
  //message can be "3-666|ooops" or just "ooops" => 3 is level, 666 is status code, remains the text description.

  //check if meta info or just a description
  const info = error.message.split("|");
  const hasMeta = info.length > 1;

  //get meta info and user friendly message
  const meta = hasMeta ? info[0] : "";
  const text = hasMeta ? info[1] : error.message;

  //extract level and code from the infos
  const name = meta.split("-");
  const level = isNaN(name[0]) ? 3 : parseInt(name[0], 10);
  const code = isNaN(name[1]) ? 0 : parseInt(name[1], 10);

  //do nothing when zero code status
  if (code === 0) return;

  return prompt(
    text || nls("core.error.default"),
    null,
    messageLevelMap[level] || "danger"
  );
}
