import { nls } from "zen-lingual";
import fetch from "./fetch";
import { selectConfig, selectToken, getZenPort } from "../utils/core";
import { logInternalError } from "../actions/notifications";
import { disconnectUser } from "../actions/auth";
import { goto } from "../actions/router";

const HttpResourceWildcard = "*";
const LocalhostWildcard = "::1";

/** Compute web-api complete uniform resource location (URL)
  @param state : redux state tree
  @param endpoint : relative web service endpoint
  @param restricted : true if web-service needs authentication token
*/
export const computeURL = (state, endpoint, restricted = false) => {
  let url = endpoint;

  if (url.indexOf(HttpResourceWildcard) === 0) {
    //case endpoint is a HTTP served resource (meaning no API call)
    url = url.replace(HttpResourceWildcard, "");
  } else if (url.indexOf(LocalhostWildcard) === 0) {
    //case endpoint is a local HTTP server
    url = url.replace(
      LocalhostWildcard,
      `http://127.0.0.1:${getZenPort(state)}`
    );
  } else {
    //only add API root when necessary
    const apiRoot = selectConfig(state).webAPI;
    url =
      apiRoot && url.indexOf(apiRoot) === -1 ? apiRoot + endpoint : endpoint;
  }

  if (restricted) {
    //add token for restricted API
    url += "/?token=" + selectToken(state);
  }
  return url;
};

//mapping from CRUD method actions to HTTP verbs
const methodMap = {
  CREATE: "POST",
  UPDATE: "PUT",
  PATCH: "PATCH",
  DELETE: "DELETE",
  READ: "GET"
};

//WEB-API execution (calling)
const callWebApi = (state, endpoint, method, data, options) => {
  //compute final URL
  const url = computeURL(state, endpoint, false);
  //and now fetch
  return fetch(
    url,
    methodMap[method],
    data,
    options,
    options.restricted ? selectToken(state) : ""
  );
};

/** Redux middleware that interprets actions with 'API' info specified.
  Provides an extension point between dispatching an action, and the moment it reaches the reducer.
  Performs the web api call and returns promises.
  @param {function} store - redux state tree storage
  @param {function} next - next middleware's dispatch function
  @param {object} action - current dispatched action
*/
export default store => next => action => {
  const { api } = action;
  //run next action if not an API action
  if (!api) return next(action);

  //extract info from action
  const { endpoint, method = "READ", options } = api;

  if (typeof endpoint !== "string")
    throw new Error("Specify a string endpoint URL.");

  //merge options with default ones
  const finalOptions = Object.assign(
    { restricted: true, mute: false },
    options
  );

  //compute final action
  function actionWith(kind, data) {
    //create next action with its new type : FSA-TYPE_METHOD_(REQUEST|RESPONSE|FAILURE)
    const finalAction = Object.assign({}, action, data, {
      type: `${action.type}_${method}_${kind}`
    });
    //remove api to avoid recursive loops
    delete finalAction.api;
    return finalAction;
  }

  //inform that asynchronous action has started
  next(actionWith("REQUEST", {}));

  //call web api that will return a thenable
  return (
    callWebApi(store.getState(), endpoint, method, action.payload, finalOptions)
      //when response comes back
      .then(response => {
        next(
          actionWith("RESPONSE", {
            payload: response
          })
        );
        return response;
      })
      //when failure
      .catch(exc => {
        //trigger failed action
        next(
          actionWith("FAILURE", {
            error: true,
            payload: exc
          })
        );

        if (exc.number === 401) {
          //unauthorized : revoke access
          next(disconnectUser(nls("core.error.unauthorizedAccess")));
        } else if (
          exc.number === 403 ||
          exc.number === 404 ||
          exc.number === 500
        ) {
          //navigate to error pages
          next(goto(`/pages/${exc.number}`, true));
        } else {
          //logs internal error if necessary
          const log = logInternalError(exc);
          if (log) next(log);
        }

        //mute or continue rejection to ensure reject status can be 'catch' in user land call
        return finalOptions.mute ? true : Promise.reject(exc);
      })
  );
};
